from glob import glob
from os import path

try:
    from ._keccak import ffi
except ImportError:
    raise RuntimeError("Required CFFI extension not found. You need to install this package before use. See README.")


try:
    obj_name = glob(path.abspath(path.join(path.dirname(__file__), "sha3*")))[0]
except IndexError:
    raise RuntimeError("Required sha3 extension not found. You need to install this package before use. See README.")

lib = ffi.dlopen(obj_name)


class sha3_256(object):

    def __init__(self, seed=""):
        self.seed = seed

    def digest(self):
        return _sha3_256(self.seed)

    def hexdigest(self):
        return _sha3_256(self.seed).encode('hex')

    def update(self, string):
        self.seed = self.seed + string


class sha3_512(object):

    def __init__(self, seed=""):
        self.seed = seed

    def digest(self):
        return _sha3_512(self.seed)

    def hexdigest(self):
        return _sha3_512(self.seed).encode('hex')

    def update(self, string):
        self.seed = self.seed + string


# SHA3-256 hashing using the Keccak standard
def _sha3_256(seed):
    # Length of the output of sha256
    output_length = 32

    # ffi definition of the output uint array
    output = ffi.new("uint8_t[]", output_length)

    input_ = ffi.new("uint8_t[]", str(seed))

    lib.sha3_256(
        output,
        output_length,
        input_,
        len(seed)
    )

    buf = ffi.buffer(output, output_length)
    return buf[:]


# SHA3-512 hashing using the Keccak standard
def _sha3_512(seed):
    # Length of the output of sha512
    output_length = 64

    # ffi definition of the output uint array
    output = ffi.new("uint8_t[]", output_length)

    input_ = ffi.new("uint8_t[]", str(seed))

    lib.sha3_512(
        output,
        output_length,
        input_,
        len(seed)
    )

    buf = ffi.buffer(output, output_length)
    return buf[:]



from setuptools import setup, Extension, find_packages

sha3 = Extension(
    'keccak.sha3',
    sources=['lib/sha3.c'],
    depends=['lib/compiler.h', 'lib/sha3.h'],
    extra_compile_args=["-Isrc/", "-std=gnu99", "-Wall"]
)


setup(
    name="pykeccak",
    version='0.2.1',
    description="Keccak 256 hashing for PyPy2",
    author="Jacob Stenum Czepluch",
    author_email="j.czepluch@gmail.com",
    url="https://github.com/czepluch/pykeccak",
    license="MIT",
    packages=find_packages(exclude=["_cffi_build", "_cffi_build/*"]),
    ext_modules=[sha3],
    install_requires=["cffi>=1.2.1"],
    setup_requires=["cffi>=1.2.1"],
    cffi_modules=["_cffi_build/keccak_build.py:ffi"],
    classifiers=[
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: Implementation :: PyPy"
    ],
    zip_safe=False
)



from cffi import FFI


ffi = FFI()

ffi.set_source("keccak._keccak", None)
ffi.cdef(
    '''
    int sha3_256(uint8_t*, size_t, uint8_t const*, size_t);


    int sha3_512(uint8_t*, size_t, uint8_t const*, size_t);
    ''')


if __name__ == '__main__':
    ffi.compile()
